require 'concurrent-ruby'
require_relative 'pubsub'
require_relative 'metrics_collector'

module BackRun
  class Worker
    attr_reader :queues

    def initialize(pubsub, queues, threads, pool = nil)
      @queues = queues
      @pubsub = pubsub
      @pool = pool || Concurrent::ThreadPoolExecutor.new(
        min_threads: 1, max_threads: threads, max_queue: 1
      )
    end

    def start_listening!
      @pubsub.subscribe(self)
      sleep
    end

    def message_received(job, ack, modify_ack_deadline)
      if job.should_run_now?
        ack.call
        run_job(job)
      else
        remaining_seconds = job.remaining_seconds_to_run
        modify_ack_deadline.call(remaining_seconds)
      end
    rescue Concurrent::RejectedExecutionError
      BackRun.logger.info('Thread pool busy. Republishing the job')
      retry_job(job)
    end

    def retry_job(job)
      @pubsub.publish(job)
    end

    def kill_job(job)
      @pubsub.kill_job(job)
    end

    private

    def run_job(job)
      @pool.post do
        job.perform(self)
        MetricsCollector.job_executed(job)
      end
    end
  end
end
